module Metamorphosis
  class UnitValue
    include Comparable

    attr_reader :unit_symbol, :value, :exponent

    def initialize(unit_symbol, value, exponent = 1.0)
      @unit_symbol = unit_symbol.to_sym
      @value = value
      @exponent = exponent.to_f
    end

    def to_s
      "#{self.to_f} #{unit_symbol}"
    end

    def to_f
      @exponent * @value
    end

    def to_i
      to_f.to_i
    end

    def <=>(other_unit)
      unless other_unit.is_a?(UnitValue) || other_unit.is_a?(Numeric)
        raise ArgumentError,
              "Unable to compare #{self.to_s} with #{other_unit.class}"
      end

      # Assume numeric values are of the same unit type
      return self.to_f <=> other_unit if other_unit.is_a?(Numeric)

      # Convert the unit if they're not the same unit already
      unless @unit_symbol == other_unit.unit_symbol
        other_unit = other_unit.to(@unit_symbol)
      end

      self.to_f <=> other_unit.to_f
    end

    def to(unit_symbol = nil)
      return self unless unit_symbol
      Metamorphosis.engine.convert_to(unit_symbol, self)
    end
    alias_method :as, :to

    def method_missing method_name, *args, &block
      self.to(method_name) ||
        super( method_name, *args, &block )
    end

  end
end
